@@ -1,10 +1,10 @@
-make.Fterm <- function(ncoef, Ufd, factor=1) {
+make.Fterm <- function(ncoef, Ufd, factor=1, name=NULL) {
   if (floor(ncoef) != ncoef || ncoef < 1) {
     stop("Argument NCOEF is not a positive integer.")
   }
   if (!is.fd(Ufd)) {
     stop("Argument UFD is not a functional data object.")
   }
-  termList <- list(ncoef=ncoef, Ufd=Ufd, factor=factor)
+  termList <- list(ncoef=ncoef, Ufd=Ufd, factor=factor, name=name)
   return(termList)
 }
\ No newline at end of file
